Function Get-FolderName
{
<#
.SYNOPSIS
   Show a Folder Browser Dialog and return the directory selected by the user

.DESCRIPTION
  Show a Folder Browser Dialog and return the directory selected by the user

.PARAMETER SelectedPath
   Initial Directory for browsing
   Mandatory - [string]

.PARAMETER Description
   Message Box Title
   Optional - [string] - Default : "Select a Folder"

.PARAMETER  ShowNewFolderButton
   Show New Folder Button when unused (default) or doesn't show New Folder when used with $false value
   Optional - [Switch]

 .EXAMPLE
   Get-FolderName
    cmdlet Get-FileFolder at position 1 of the command pipeline
    Provide values for the following parameters:
    SelectedPath: C:\temp
    C:\Temp\

   Choose only one Directory. It's possible to create a new folder (default)

.EXAMPLE
   Get-FolderName -SelectedPath c:\temp -Description "Select a folder" -ShowNewFolderButton
   C:\Temp\Test

   Choose only one Directory. It's possible to create a new folder

.EXAMPLE
   Get-FolderName -SelectedPath c:\temp -Description "Select a folder"
   C:\Temp\Test
   Choose only one Directory. It's not possible to create a new folder

.EXAMPLE
   Get-FolderName  -SelectedPath c:\temp
   C:\Temp\Test

   Choose only one Directory. It's possible to create a new folder (default)


.EXAMPLE
 Get-Help Get-FolderName -Full

.INPUTS
   System.String
   System.Management.Automation.SwitchParameter

.OUTPUTS
   System.String


.NOTES
  Version         : 1.0
  Author          : O. FERRIERE
  Creation Date   : 12/10/2019
  Purpose/Change  : Initial development

  Based on different pages :
   mainly based on https://blog.danskingdom.com/powershell-multi-line-input-box-dialog-open-file-dialog-folder-browser-dialog-input-box-and-message-box/
   https://code.adonline.id.au/folder-file-browser-dialogues-powershell/
   https://thomasrayner.ca/open-file-dialog-box-in-powershell/
   https://code.adonline.id.au/folder-file-browser-dialogues-powershell/
#>

[CmdletBinding()]
    [OutputType([string])]
    Param
    (
        # InitialDirectory help description
        [Parameter(
            Mandatory = $true,
            ValueFromPipelineByPropertyName = $true,
            HelpMessage = "Initial Directory for browsing",
            Position = 0)]
        [String]$SelectedPath,

        # Description help description
        [Parameter(
            Mandatory = $false,
            ValueFromPipelineByPropertyName = $true,
            HelpMessage = "Message Box Title")]
        [String]$Description="Select a Folder",

        # ShowNewFolderButton help description
        [Parameter(
            Mandatory = $false,
            HelpMessage = "Show New Folder Button when used")]
        [Switch]$ShowNewFolderButton
    )

    # Load Assembly
    Add-Type -AssemblyName System.Windows.Forms

    # Open Class
    $FolderBrowser= New-Object System.Windows.Forms.FolderBrowserDialog

   # Define Title
    $FolderBrowser.Description = $Description

    # Define Initial Directory
    if (-Not [String]::IsNullOrWhiteSpace($SelectedPath))
    {
        $FolderBrowser.SelectedPath=$SelectedPath
    }

    if($folderBrowser.ShowDialog() -eq "OK")
    {
        $Folder += $FolderBrowser.SelectedPath
    }
    return $Folder
}